using System;
using System.Collections.Generic;

namespace Calculator
{
	/// <summary>
	/// Models a simple RPN calculator
	/// </summary>
	public class ReversePolishNotationCalculator
	{
		private Stack<INode> innerStack;
		
		private Evaluator evaluator;
		/// <summary>
		/// Constructor for the calculator
		/// </summary>
		public ReversePolishNotationCalculator ()
		{
			innerStack = new Stack<INode>();
			evaluator = new Evaluator(innerStack);
		}
		/// <summary>
		/// Return the stack used to model the rpn calculator
		/// </summary>
		/// <returns>
		/// A <see cref="Stack<INode>"/>
		/// </returns>
		public Stack<INode> GetStack()
		{
			return innerStack;
		}
		/// <summary>
		/// Is the internal stack empty?
		/// </summary>
		/// <returns>
		/// A <see cref="Boolean"/>
		/// </returns>
		public Boolean IsStackEmpty()
		{
			return innerStack.Count == 0;				
		}	
		/// <summary>
		/// Push "+" on top of the stack
		/// </summary>
		public void PushPlusOperator()
		{
			innerStack.Push(new PlusOperator());
		}
		/// <summary>
		/// Push "-" on top of the stack
		/// </summary>
		public void PushMinusOperator()
		{
			innerStack.Push(new MinusOperator());
		}
		/// <summary>
		/// Push "*" on top of the stack
		/// </summary>
		public void PushTimesOperator()
		{
			innerStack.Push(new TimesOperator());
		}
		/// <summary>
		/// Push "/" on top of the stack
		/// </summary>
		public void PushDivideByOperator()
		{
			innerStack.Push(new DivideByOperator());
		}
		/// <summary>
		/// Push an operand on top of the stack
		/// </summary>
		/// <param name="op">
		/// A <see cref="System.Double"/>
		/// </param>
		public void PushOperand(double op)
		{
			innerStack.Push(new Operand(op));
		}
		/// <summary>
		/// Pop an operand from the top of the stack
		/// </summary>
		/// <returns>
		/// A <see cref="System.Double"/>
		/// </returns>
		public double PopOperand()
		{
			return ((Operand)(innerStack.Pop())).InternalValue;						
		}
		/// <summary>
		/// Peek an operand on the top of the stack (don't remove it) 
		/// </summary>
		/// <returns>
		/// A <see cref="System.Double"/>
		/// </returns>
		public double PeekOperand()
		{
			return ((Operand)(innerStack.Peek())).InternalValue;					
		}
		/// <summary>
		/// Evaluate the stack
		/// </summary>
		public void EvaluateStack()
		{ 
			if (IsStackEmpty())
			{
				throw new EmptyStackEvaluationException();
			}
			INode n = innerStack.Pop();
			n.Accept (evaluator);
			innerStack.Push(new Operand(evaluator.Result));		
		}
	}
}
